Package: HLAtools
Type: Package
Title: Toolkit for HLA Immunogenomics
Version: 1.11.0
Authors@R: 
    c(person("Livia", "Tran", , "livia.tran@ucsf.edu", role = c("aut")),
    person("Ryan", "Nickens", , "rynickens@gmail.com", role = c("aut")),
    person("Leamon", "Crooms IV", , "lcroomsiv@gmail.com", role = "aut"),
    person("Derek", "Pappas", , "djpappas75@gmail.com", role = "aut"),
    person("Vinh", "Luu", , "vinhluu864@berkeley.edu", role = "ctb"),
    person("Josh", "Bredeweg", , role = "ctb"),
    person("Steven", "Mack", , "Steven.Mack@ucsf.edu", role = c("aut", "cre"), 
           comment = c(ORCID = "0000-0001-9820-9547")))
Maintainer: Steven Mack <Steven.Mack@ucsf.edu>
Description: A toolkit for the analysis and management of data for genes in the so-called "Human Leukocyte Antigen" (HLA) region. Functions extract reference data from the Anthony Nolan HLA Informatics Group/ImmunoGeneTics HLA 'GitHub' repository (ANHIG/IMGTHLA) <https://github.com/ANHIG/IMGTHLA>, validate Genotype List (GL) Strings, convert between UNIFORMAT and GL String Code (GLSC) formats, translate HLA alleles and GLSCs across ImmunoPolymorphism Database (IPD) IMGT/HLA Database release versions, identify differences between pairs of alleles at a locus, generate customized, multi-position sequence alignments, trim and convert allele-names across nomenclature epochs, and extend existing data-analysis methods. Tran et al., (2025) <doi:10.1111/iji.70013>. 
Encoding: UTF-8
Imports: DescTools, dplyr, fmsb, rvest, stats, stringr, tibble, utils,
        xfun
Depends: R (>= 3.6.0)
LazyData: true
LazyDataCompression: xz
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
License: GPL (>= 3)
URL: <https://github.com/sjmack/HLAtools>
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-21 00:33:19 UTC; sjmack
Author: Livia Tran [aut],
  Ryan Nickens [aut],
  Leamon Crooms IV [aut],
  Derek Pappas [aut],
  Vinh Luu [ctb],
  Josh Bredeweg [ctb],
  Steven Mack [aut, cre] (ORCID: <https://orcid.org/0000-0001-9820-9547>)
Repository: CRAN
Date/Publication: 2026-07-22 08:10:14 UTC
Built: R 4.5.2; ; 2026-07-22 12:27:44 UTC; unix
