# Generated by roxygen2: do not edit by hand

export(BDstrat)
export(BDtoPyPop)
export(GIANT)
export(GLStoUNI)
export(GLV)
export(GLV2)
export(GLVhelper)
export(GLtoUN)
export(GLupdate)
export(GLvalidate)
export(UNItoGLS)
export(UNtoGL)
export(addCodonLine)
export(alignmentFull)
export(alignmentSearch)
export(alleleTrim)
export(atlasFull)
export(atlasMaker)
export(buildAlignments)
export(buildGazetteer)
export(buildGenomicReferences)
export(buildIMGTHLAGeneTypes)
export(checkAlignType)
export(checkSource)
export(checkVersion)
export(checkgDNAstart)
export(compareSequences)
export(convertAny)
export(countSpaces)
export(customAlign)
export(expandVersion)
export(ffN)
export(formatHead)
export(getAlignmentNames)
export(getField)
export(getLatestVersion)
export(motifMatch)
export(multiAlign)
export(multiAlleleTrim)
export(multiGLStoUNI)
export(multiLocusValidation)
export(multiQueryRelease)
export(multiSearch)
export(multiTranslateGLstring)
export(multiUNItoGLS)
export(multiUpdateGL)
export(numFields)
export(parseAlignmentHead)
export(posSort)
export(pypopHeaders)
export(queryPositions)
export(queryRelease)
export(redec)
export(relRisk)
export(repoVersion)
export(squashVersion)
export(translateAllele)
export(translateGLstring)
export(typeToSource)
export(uniAlign)
export(uniSearch)
export(updateAll)
export(updateAlleleListHistory)
export(updateGL)
export(validateAllele)
export(validateGLstring)
export(validateLocus)
export(validateMotif)
export(validatePositions)
export(validateUniformat)
export(validateVersion)
export(variantTable)
export(verifyAllele)
import(dplyr)
importFrom(DescTools,IsOdd)
importFrom(DescTools,StrLeft)
importFrom(dplyr,"%>%")
importFrom(dplyr,filter)
importFrom(fmsb,riskratio)
importFrom(rvest,html_elements)
importFrom(rvest,html_nodes)
importFrom(rvest,html_table)
importFrom(rvest,read_html)
importFrom(stats,setNames)
importFrom(stringr,"%>%")
importFrom(stringr,fixed)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_squish)
importFrom(tibble,add_column)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
importFrom(xfun,numbers_to_words)
