#declaring global variables
utils::globalVariables(".")
utils::globalVariables("HLAalignments")
utils::globalVariables("alleleListHistory")
utils::globalVariables("HLAgazetteer")
utils::globalVariables("fragmentFeatureNames")
utils::globalVariables("IMGTHLAGeneTypes")
utils::globalVariables("HLAatlas")
utils::globalVariables("IMGTGenomicReferences")