% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VersionValidation.R
\name{getLatestVersion}
\alias{getLatestVersion}
\title{Identify the Latest IPD-IMGT/HLA Database Release}
\usage{
getLatestVersion()
}
\value{
A dot-delimited character string identifying the latest release version (branch) of the ANHIG/IMGTHLA Github repository.
If the most recent release version is not a 'primary' release version (i.e., following the X.YY.O naming format), the integer
in the third field of the release version is replaced with a '0'.
}
\description{
Identifies the most recent version of the IPD-IMGT/HLA Database available in the ANHIG/IMGTHLA Github repository.
}
\note{
For internal HLAtools use.
}
\examples{
getLatestVersion()

}
